import 'package:flutter/material.dart';

// import 'package:modal_progress_hud/modal_progress_hud.dart';
import 'package:modal_progress_hud_nsn/modal_progress_hud_nsn.dart';
import 'package:payment_gateway/AppHeader.dart';
import 'package:payment_gateway/constants/strings.dart';
import 'package:payment_gateway/constants/utility.dart';
import 'package:payment_gateway/logic/safex_pay_bloc.dart';
import 'package:payment_gateway/login_screen.dart';
import 'package:payment_gateway/observer/safexpay_observable.dart';

import 'NetBanking.dart';
import 'PaymentMode.dart';
import 'cardDetails.dart';
import 'observer/safexpay_observer.dart';

class SafeXPayGateway extends StatefulWidget {
  SafeXPayGateway(
      {required this.orderNo,
      required this.amount,
      required this.currency,
      required this.transactionType,
      required this.channel,
      required this.successUrl,
      required this.failureUrl,
      required this.countryCode});

  final String orderNo;
  final double amount;
  final String currency;
  final String transactionType;
  final String channel;
  final String successUrl;
  final String failureUrl;
  final String countryCode;

  @override
  _SafeXPayGatewayState createState() => _SafeXPayGatewayState();
}

class _SafeXPayGatewayState extends State<SafeXPayGateway> implements SafeXPayObserver {
  late SafeXPayBloc safeXPayBloc;
  late List<Widget> _children;
  int currentIndex = 0;
  SafeXPayObservable _safeXPayObservable = SafeXPayObservable();

  @override
  void initState() {
    _safeXPayObservable.register(this);
    safeXPayBloc = SafeXPayBloc(
        orderNo: widget.orderNo,
        amount: widget.amount,
        currency: widget.currency,
        transactionType: widget.transactionType,
        channel: widget.channel,
        successUrl: widget.successUrl,
        failureUrl: widget.failureUrl,
        countryCode: widget.countryCode);

    getMerchantBrandingDetails();
    generateChildrenList();
    super.initState();

  }

  Future<void> getMerchantBrandingDetails() async {
    bool isInternetAvailable = await Utility.isNetworkAvailable();

    if (isInternetAvailable) {
      safeXPayBloc.getBrandingDetails();
      //safeXPayBloc.getCardType('607385');
    } else {
      Utility.ackAlert(
          context, StringConstants.INTERNET_CONNECTIVITY_ERROR_MESSAGE);
    }
  }

  void generateChildrenList() {
    _children = <Widget>[
      LoginScreen(safeXPayBloc: safeXPayBloc),
      PaymentMode(safeXPayBloc: safeXPayBloc),
      CardDetails(safeXPayBloc: safeXPayBloc),
      NetBanking(safeXPayBloc: safeXPayBloc),
      // LoginScreen(safeXPayBloc: safeXPayBloc),
    ];
  }

  @override
  void dispose() {
    _safeXPayObservable.unRegister(this);
    safeXPayBloc.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: safeXPayBloc.scaffoldKey,
      body: StreamBuilder<bool>(
          stream: safeXPayBloc.loaderController.stream,
          builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
            bool inAsyncCall = false;
            if (snapshot.hasData && snapshot.data != null) {
              inAsyncCall = snapshot.data!;
            }
            return ModalProgressHUD(
              inAsyncCall: inAsyncCall,
              child: Column(
                children: [
                  AppHeader(
                    amount: widget.amount,
                    currency: widget.currency,
                    orderNo: widget.orderNo,
                  ),
                  /*Expanded(
                    child: MaterialApp(
                      debugShowCheckedModeBanner: false,
                      home: LoginScreen(
                        safeXPayBloc: safeXPayBloc,
                      ),
                    ),
                  ),*/
                  Expanded(
                    child: IndexedStack(
                      index: currentIndex,
                      children: _children,
                    ),
                  )
                ],
              ),
            );
          }),
    );
  }

  @override
  void onIndexChanged(int index) {
    setState(() {
      currentIndex = index;
    });
  }
}
